"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-05-20
Author: Martian Bugs
Description: 商品分析模块
"""

from BrowserAutomationLauncher._browser import Browser

from .brand import Brand
from .ranking import Ranking


class GoodsAnalysis:
    def __init__(self, browser: Browser):
        self._browser = browser
        self._ranking = None
        self._brand = None

    @property
    def ranking(self):
        """商品排行数据采集"""

        if not self._ranking:
            self._ranking = Ranking(self._browser)
        return self._ranking

    @property
    def brand(self):
        """品牌分析数据采集"""

        if not self._brand:
            self._brand = Brand(self._browser)
        return self._brand



class PostData:
    goods_ranking_export = {
        # 'beginDate': '2025-05-20',
        # 'endDate': '2025-05-20',
        'period': '1d',
        'dataType': 1,
        'dataType1': 1,
        'supplier': [],
        'brand': ['639ac2710ac4db00013decc7'],
        'category': '',
        'level': 0,
        'keywords': '',
        'items': [
            'sku_concat_name',
            'brand_name',
            'trd_cnt',
            'per_qty_price',
            'trd_discount_rto',
            'per_usr_trd_qty',
            'per_trd_sku_qty',
            'ord_uv_val',
            'save_click_uv',
            'per_usr_visit_pdtl_pv',
            'inventory_qty',
            'trd_gf_sku_qty',
            'expo_uv',
            'visit_pdtl_uv',
            'per_usr_vist_pdtl_stay_dur',
            'visit_pdtl_rto',
            'add_card_uv',
            'add_cart_uv_rto',
            'trd_usr_num',
            'trd_qty',
            'trd_amt',
            'trd_lfapp_expo_uv_tsfm_rto',
        ],
        'sortItem': 'expo_uv desc',
    }
    brand_channel_export = {
        # 'beginDate': '2025-05-19',
        # 'endDate': '2025-05-19',
        'period': '1d',
        'dataType': 1,
        'supplier': [],
        'brand': ['639ac2710ac4db00013decc7'],
        'items': [
            'usr_atv',
            'per_usr_trd_qty',
            'per_trd_sku_qty',
            'expo_uv',
            'visit_pdtl_uv',
            'visit_pdtl_rto',
            'add_card_uv',
            'add_cart_uv_rto',
            'trd_usr_num',
            'trd_qty',
            'trd_amt',
            'trd_lfapp_expo_uv_tsfm_rto',
            'trd_cnt',
        ],
        'sortItem': 'expo_uv desc',
    }



"""
品牌分析数据采集
"""

from copy import deepcopy
from tempfile import gettempdir
from time import sleep
from typing import Callable

from BrowserAutomationLauncher import DataPacketProcessor
from BrowserAutomationLauncher._browser import Browser
from BrowserAutomationLauncher._utils.tools import DateTimeTools, DictUtils, OsTools
from DrissionPage._pages.mix_tab import MixTab

from .._types import PUB_WEB_REQUEST_HEADERS
from ._types import PostData
from ._utils import generate__h_enc, pick__date


class Urls:
    brand = 'https://business-decision.cdfsunrise.com/business_decision/analysis/brandAnalysis'


class DataPacketUrls:
    brand_info = 'https://business-decision.cdfsunrise.com/base-api/report/brand/info'
    """品牌信息数据"""
    channel_detail = (
        'https://business-decision.cdfsunrise.com/base-api/report/brand/channel'
    )
    """品牌+场域表格数据"""
    channel_detail__export = (
        'https://business-decision.cdfsunrise.com/base-api/report/brand/channel/export'
    )
    """品牌+场域表格数据导出"""


class Brand:
    def __init__(self, browser: Browser):
        self._browser = browser
        self._timeout = 15

    def __format__data_list(self, data_list: list[dict]):
        """格式化数据列表"""

        _data_list = []
        for item in deepcopy(data_list):
            temp = DictUtils.dict_format__number(item, exclude_fields=['场域'])
            temp = DictUtils.dict_format__ratio(
                temp,
                fields=['商详到达率（日均）', '加购率（日均）', '购买转化率（日均）'],
            )
            _data_list.append(temp)

        return _data_list

    def _download_report(
        self, page: MixTab, timeout: float = None, download_btn_index: int = None
    ) -> str:
        """
        下载报表

        Args:
            page: 页面对象
            timeout: 超时时间
            download_btn_index: 下载按钮的序号, 默认为 0
        Returns:
            表格文件路径
        """

        trigger_btns = page.eles('t:div@@class=rs-btn@@text()=下载为Excel', timeout=1)
        if not trigger_btns:
            raise RuntimeError('未找到 [下载为Excel] 按钮')
        trigger_btn = trigger_btns[download_btn_index or 0]
        trigger_btn.click(by_js=True)

        drawer = page.ele('c:div.ant-drawer-content', timeout=3)
        if not drawer:
            raise RuntimeError('未找到 [下载抽屉]')

        sleep(0.5)
        select_all_btn = drawer.ele(
            'c:div.ant-transfer-list-header label.ant-transfer-list-checkbox', timeout=3
        )
        if not select_all_btn:
            raise RuntimeError('未找到 [全选] 按钮')
        select_all_btn.click(by_js=True)

        sleep(0.5)
        transfer_arrow_btn = drawer.ele(
            'c:button.ant-btn span.anticon-right', timeout=1
        )
        if not transfer_arrow_btn:
            raise RuntimeError('未找到 [右箭头] 按钮')
        transfer_arrow_btn.click(by_js=True)

        download_btn = drawer.ele('t:div@@class=rs-btn@@text()=下载', timeout=1)
        if not download_btn:
            raise RuntimeError('未找到 [下载] 按钮')

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout
        mission = download_btn.click.to_download(
            save_path=gettempdir(),
            by_js=True,
            timeout=_timeout,
        )
        file_path: str = mission.wait(show=False)
        return file_path

    def get__channel_detail(self, date: str, raw=False, timeout: float = None):
        """
        获取场域表格详情数据

        Args:
            date: 日期
            raw: 为 True 则返回下载的文件路径, 否则返回表格数据
            timeout: 超时时间
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout
        page = self._browser.chromium.new_tab(Urls.brand)

        # 切换到 品牌+场域 标签页
        channel_tab = page.ele('#rc-tabs-1-tab-3', timeout=5)
        if not channel_tab:
            raise RuntimeError('未找到 [品牌+场域] 标签页')

        def wait__detail_datapacket(callback: Callable):
            page.listen.start(
                targets=DataPacketUrls.channel_detail, method='POST', res_type='XHR'
            )
            callback()
            datapacket = page.listen.wait(timeout=_timeout)
            return datapacket

        detail_datapacket = wait__detail_datapacket(
            lambda: channel_tab.click(by_js=True)
        )
        if not detail_datapacket:
            raise TimeoutError('进入页面后获取表格详情数据包超时')

        detail_datapacket__processor = DataPacketProcessor(detail_datapacket)
        detail_datapacket__data = detail_datapacket__processor.filter('?content.total')
        if not detail_datapacket__data.get('total'):
            raise ValueError('表格详情数据为空')

        is_yeasterday = DateTimeTools.date_yesterday() == date
        if not is_yeasterday and not wait__detail_datapacket(
            lambda: pick__date(date, page)
        ):
            raise RuntimeError('修改日期后获取表格详情数据包超时')

        try:
            report_file_path = self._download_report(page, download_btn_index=1)
        except Exception as e:
            raise RuntimeError(f'下载报表失败: {e}') from e

        page.close()

        if raw is True:
            return report_file_path

        data_list = OsTools.xlsx_read(file_path=report_file_path)
        OsTools.file_remove(report_file_path)

        data_list = self.__format__data_list(data_list)
        return data_list

    def get__channel_detail__by_api(
        self, begin_date: str, end_date: str, raw=False, timeout: float = None
    ):
        """
        通过 API 获取品牌+场域表格数据

        Args:
            begin_date: 开始日期
            end_date: 结束日期
            raw: 为 True 则返回下载的文件路径, 否则返回表格数据
            timeout: 超时时间
        Returns:
            表格数据或表格文件的路径
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout

        page = self._browser.chromium.new_tab()
        page.listen.start(
            targets=DataPacketUrls.brand_info, method='POST', res_type='XHR'
        )
        page.get(Urls.brand)
        detail_datapacket = page.listen.wait(timeout=_timeout)
        if not detail_datapacket:
            raise TimeoutError('进入页面后获取品牌信息数据包超时')

        auth_token = detail_datapacket.request.headers.get('auth-token')
        post_data = {
            'beginDate': begin_date,
            'endDate': end_date,
            **PostData.brand_channel_export,
        }
        post_data_str, h_enc = generate__h_enc(
            auth_token, DataPacketUrls.channel_detail__export, post_data
        )
        headers = {
            **PUB_WEB_REQUEST_HEADERS,
            'H-Enc': h_enc,
            'Auth-Token': auth_token,
            'Content-Type': 'application/json',
        }

        page.change_mode('s', go=False)
        resp = page.post(
            DataPacketUrls.channel_detail__export,
            data=post_data_str,
            headers=headers,
            timeout=_timeout,
        )
        resp_data: dict = resp.json()
        if resp_data.get('code') != 20000:
            errmsg = resp_data.get('message')
            raise RuntimeError(f'表格下载出错: {errmsg}')

        download_url = resp_data.get('content')
        download_state, file_path = page.download(
            file_url=download_url,
            save_path=gettempdir(),
            file_exists='overwrite',
            timeout=_timeout,
            show_msg=False,
        )
        if download_state != 'success':
            raise RuntimeError('表格下载失败')

        page.change_mode('d', go=False)
        page.close()

        if raw is True:
            return file_path

        data_list = OsTools.xlsx_read(file_path=file_path)
        OsTools.file_remove(file_path)

        data_list = self.__format__data_list(data_list)
        return data_list



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-05-19
Author: Martian Bugs
Description: 后台登录
"""

from os import path
from tempfile import gettempdir
from time import sleep

from BrowserAutomationLauncher._browser import Browser
from BrowserAutomationLauncher._utils.tools import OsTools
from ddddocr import DdddOcr


class Urls:
    home = 'https://business-decision.cdfsunrise.com/'


class DataPacketUrls:
    captcha_img = (
        'https://uoc-gateway-lion.cdfsunrise.com/admin-api/system/captcha/generate'
    )
    # 图形验证码生成接口
    login = 'https://uoc-gateway-lion.cdfsunrise.com/admin-api/system/auth/sso-login'


class Login:
    def __init__(self, browser: Browser):
        self._browser = browser

    def _parse_captcha_img(self, img_path: str):
        """
        识别图形验证码

        Args:
            img_path: 验证码图片路径
        Returns:
            验证码字符串
        """

        if not path.exists(img_path):
            raise FileNotFoundError(f'验证码图片 {img_path} 不存在')

        try:
            ocr = DdddOcr(show_ad=False)
            code = None
            with open(img_path, 'rb') as f:
                code = ocr.classification(f.read())

            OsTools.file_remove(img_path)
            return code
        except Exception as e:
            raise RuntimeError(f'图形验证码识别出错: {e}') from e

    def by_password(self, username: str, password: str):
        """
        通过账号密码登录

        Args:
            username: 登录名
            password: 密码
        """

        cache_token_key = 'Y2FjaGUtdG9rZW4='

        page = self._browser.chromium.new_tab()
        page.listen.start(
            targets=DataPacketUrls.captcha_img, method='GET', res_type='XHR'
        )
        page.get(Urls.home)

        if page.local_storage(cache_token_key):
            sleep(5)
            if page.local_storage(cache_token_key):
                page.listen.stop()
                return

        captcha_datapacket = page.listen.wait(timeout=8)
        if not captcha_datapacket:
            raise TimeoutError('图形验证码获取超时')

        captcha_img_path = path.join(gettempdir(), 'cdfsunrise_login_captcha.png')
        with open(captcha_img_path, 'wb') as f:
            f.write(captcha_datapacket.response.body)

        captcha_code = self._parse_captcha_img(captcha_img_path)
        if not captcha_code or not isinstance(captcha_code, str):
            raise ValueError('图形验证码识别失败')

        username_input = page.ele('#username', timeout=3)
        if not username_input:
            raise RuntimeError('未找到 [用户名] 输入框')
        username_input.input(username, clear=True)

        password_input = page.ele('#password', timeout=1)
        if not password_input:
            raise RuntimeError('未找到 [密码] 输入框')
        password_input.input(password, clear=True)

        captcha_input = page.ele('#code', timeout=1)
        if not captcha_input:
            raise RuntimeError('未找到 [验证码] 输入框')
        captcha_input.input(captcha_code, clear=True)

        login_btn = page.ele('t:button@@text()=登 录', timeout=1)
        if not login_btn:
            raise RuntimeError('未找到 [登录] 按钮')

        page.listen.start(targets=DataPacketUrls.login, method='POST', res_type='XHR')
        login_btn.click(by_js=True)
        login_datapacket = page.listen.wait(timeout=15)
        login_datapacket_data = login_datapacket.response.body
        if isinstance(login_datapacket_data, dict) and (
            login_msg := login_datapacket_data.get('msg')
        ):
            raise RuntimeError(f'登录失败: {login_msg}')

        for _ in range(15):
            cache_token = page.local_storage(cache_token_key)
            if cache_token:
                break
            sleep(1)
        else:
            raise TimeoutError('登录超时, 未找到 cache-token 值')



import json
from base64 import b64encode
from hashlib import md5

from BrowserAutomationLauncher._utils.tools import DateTimeTools
from DrissionPage._pages.mix_tab import MixTab


def pick__date(date: str, page: MixTab):
    """
    选择指定日期

    Args:
        date: 日期字符串，格式为'YYYY-MM-DD'
        page: MixTab 页面对象
    """

    trigger_btn = page.ele(
        't:div@@class=ant-segmented-item-label@@text()=日', timeout=1
    )
    if not trigger_btn:
        raise RuntimeError('未找到日期选择按钮')
    trigger_btn.click()

    date_picker__panel = page.ele('c:div.ant-picker-panel', timeout=3)
    if not date_picker__panel:
        raise RuntimeError('未找到日期选择面板')

    target_date_btn = date_picker__panel.ele(f'c:td[title="{date}"]', timeout=1)
    if target_date_btn:
        target_date_btn.click(by_js=True)
        return

    target_year, target_month, _ = date.split('-')
    target_year_month = f'{target_year}-{target_month}'

    curr_year_ele = date_picker__panel.ele('c:button.ant-picker-year-btn', timeout=1)
    if target_year + '年' != curr_year_ele.text:
        curr_year_ele.click(by_js=True)
        target_year_btn = date_picker__panel.ele(
            f'c:td[title="{target_year}"]', timeout=2
        )
        if not target_year_btn:
            raise RuntimeError(f'未找到年份 [{target_year}] 按钮')
        target_year_btn.click(by_js=True)

        target_month_btn = date_picker__panel.ele(
            f'c:td[title="{target_year_month}"]', timeout=2
        )
        if not target_month_btn:
            raise RuntimeError(f'未找到年月 [{target_year_month}] 按钮')
        target_month_btn.click(by_js=True)
    else:
        curr_month_ele = date_picker__panel.ele(
            'c:button.ant-picker-month-btn', timeout=1
        )
        if target_month.lstrip('0') + '月' != curr_month_ele.text:
            curr_month_ele.click(by_js=True)
            target_month_btn = date_picker__panel.ele(
                f'c:td[title="{target_year_month}"]', timeout=2
            )
            if not target_month_btn:
                raise RuntimeError(f'未找到月份 [{target_month}] 按钮')
            target_month_btn.click(by_js=True)

    target_date_btn = date_picker__panel.ele(f'c:td[title="{date}"]', timeout=1)
    if not target_date_btn:
        raise RuntimeError(f'未找到日期 [{date}] 按钮')

    target_date_btn.click(by_js=True)


def generate__h_enc(token: str, api_url: str, request_body: dict | str):
    """
    生成 H-Enc 值（用于签名请求）

    Args:
        token: 固定的 token 字符串（例如 JWT）
        api_url: 完整请求 URL，比如 "https://.../base-api/xxx"
        request_body: POST 请求体（Python dict），key 顺序须和前端 JSON.stringify 保持一致
    Returns:
        (JSON 化的请求参数, H-Enc 值（32 位小写 MD5）)
    """

    path = '/' + api_url.split('/base-api/')[1]
    t_path = b64encode(path.encode('utf-8')).decode('ascii')
    json_str = (
        json.dumps(request_body, separators=(',', ':'), ensure_ascii=False)
        if isinstance(request_body, dict)
        else request_body
    )
    t_body = b64encode(json_str.encode('utf-8')).decode('ascii')
    raw_string = (
        token + t_path + t_body + DateTimeTools.date_calculate(0, pattern='%Y%m%d')
    )
    return json_str, md5(raw_string.encode('utf-8')).hexdigest()



PUB_WEB_REQUEST_HEADERS = {
    'sec-ch-ua': '"Not(A:Brand";v="99", "Google Chrome";v="133", "Chromium";v="133"',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-platform': '"Windows"',
    'Sec-Fetch-Dest': 'empty',
    'Sec-Fetch-Mode': 'cors',
    'Sec-Fetch-Site': 'same-origin',
}



"""
商品排行模块数据采集
"""

from copy import deepcopy
from tempfile import gettempdir
from time import sleep
from typing import Callable

from BrowserAutomationLauncher import DataPacketProcessor
from BrowserAutomationLauncher._browser import Browser
from BrowserAutomationLauncher._utils.tools import DateTimeTools, DictUtils, OsTools
from DrissionPage._pages.mix_tab import MixTab

from .._types import PUB_WEB_REQUEST_HEADERS
from ._types import PostData
from ._utils import generate__h_enc, pick__date


class Urls:
    ranking = 'https://business-decision.cdfsunrise.com/business_decision/analysis/productRanking'


class DataPacketUrls:
    detail = 'https://business-decision.cdfsunrise.com/base-api/report/sku/rank'
    """表格详情数据接口"""
    favorite_detail = (
        'https://business-decision.cdfsunrise.com/base-api/report/sku/favourite/list'
    )
    """收藏表格详情数据接口"""
    export = 'https://business-decision.cdfsunrise.com/base-api/report/sku/rank/export'
    """商品表格导出接口"""
    favorite_export = 'https://business-decision.cdfsunrise.com/base-api/report/sku/favourite/list/export'
    """关注的商品表格导出接口"""


class Ranking:
    def __init__(self, browser: Browser):
        self._browser = browser
        self._timeout = 15

    def __format__data_list(self, data_list: list[dict]):
        """格式化数据列表"""

        _data_list = []
        for item in deepcopy(data_list):
            temp = DictUtils.dict_format__number(
                item, exclude_fields=['商品料号', '商品名称', '商品条形码', '品牌']
            )
            temp = DictUtils.dict_format__ratio(
                temp,
                fields=[
                    '折扣率',
                    '商详到达率(日均)',
                    '加购率(日均)',
                    '购买转化率(日均)',
                ],
            )
            _data_list.append(temp)

        return _data_list

    def _download_report(
        self, page: MixTab, timeout: float = None, download_btn_index: int = None
    ) -> str:
        """
        下载报表

        Args:
            page: 页面对象
            timeout: 超时时间
            download_btn_index: 下载按钮的序号, 默认为 0
        Returns:
            表格文件路径
        """

        trigger_btns = page.eles('t:div@@class=rs-btn@@text()=下载为Excel', timeout=1)
        if not trigger_btns:
            raise RuntimeError('未找到 [下载为Excel] 按钮')
        trigger_btn = trigger_btns[download_btn_index or 0]
        trigger_btn.click(by_js=True)

        drawer = page.ele('c:div.ant-drawer-content', timeout=3)
        if not drawer:
            raise RuntimeError('未找到 [下载抽屉]')

        sleep(0.5)
        select_all_btn = drawer.ele(
            'c:div.ant-transfer-list-header label.ant-transfer-list-checkbox', timeout=3
        )
        if not select_all_btn:
            raise RuntimeError('未找到 [全选] 按钮')
        select_all_btn.click(by_js=True)

        sleep(0.5)
        transfer_arrow_btn = drawer.ele(
            'c:button.ant-btn span.anticon-right', timeout=1
        )
        if not transfer_arrow_btn:
            raise RuntimeError('未找到 [右箭头] 按钮')
        transfer_arrow_btn.click(by_js=True)

        download_btn = drawer.ele('t:div@@class=rs-btn@@text()=下载', timeout=1)
        if not download_btn:
            raise RuntimeError('未找到 [下载] 按钮')

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout
        mission = download_btn.click.to_download(
            save_path=gettempdir(),
            by_js=True,
            timeout=_timeout,
        )
        file_path: str = mission.wait(show=False)
        return file_path

    def get__detail(
        self,
        date: str,
        raw=False,
        timeout: float = None,
    ):
        """
        获取商品排行表格数据

        Args:
            date: 指定日期
            raw: 为 True 则返回下载的文件路径, 否则返回表格数据
            timeout: 超时时间
        Returns:
            表格数据或表格文件的路径
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout
        page = self._browser.chromium.new_tab()

        def wait__detail_datapacket(callback: Callable):
            page.listen.start(
                targets=DataPacketUrls.detail, method='POST', res_type='XHR'
            )
            callback()
            datapacket = page.listen.wait(timeout=_timeout)
            return datapacket

        detail_datapacket = wait__detail_datapacket(lambda: page.get(Urls.ranking))
        if not detail_datapacket:
            raise TimeoutError('进入页面后获取表格详情数据包超时')

        detail_datapacket__processor = DataPacketProcessor(detail_datapacket)
        detail_datapacket__data = detail_datapacket__processor.filter('?content.total')
        if not detail_datapacket__data.get('total'):
            raise ValueError('表格详情数据为空')

        is_yeasterday = DateTimeTools.date_yesterday() == date
        if not is_yeasterday and not wait__detail_datapacket(
            lambda: pick__date(date, page)
        ):
            raise RuntimeError('修改日期后获取表格详情数据包超时')

        try:
            report_file_path = self._download_report(page)
        except Exception as e:
            raise RuntimeError(f'下载报表失败: {e}') from e

        page.close()

        if raw is True:
            return report_file_path

        data_list = OsTools.xlsx_read(file_path=report_file_path)
        OsTools.file_remove(report_file_path)

        data_list = self.__format__data_list(data_list)
        return data_list

    def get__detail__by_api(
        self, begin_date: str, end_date: str, raw=False, timeout: float = None
    ):
        """
        通过 API 获取商品排行表格数据

        Args:
            begin_date: 开始日期
            end_date: 结束日期
            raw: 为 True 则返回下载的文件路径, 否则返回表格数据
            timeout: 超时时间
        Returns:
            表格数据或表格文件的路径
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout

        page = self._browser.chromium.new_tab()
        page.listen.start(targets=DataPacketUrls.detail, method='POST', res_type='XHR')
        page.get(Urls.ranking)
        detail_datapacket = page.listen.wait(timeout=_timeout)
        if not detail_datapacket:
            raise TimeoutError('表格详情数据包获取超时')

        detail_datapacket__processor = DataPacketProcessor(detail_datapacket)
        detail_datapacket__data = detail_datapacket__processor.filter('?content.total')
        if not detail_datapacket__data.get('total'):
            raise ValueError('表格详情数据为空')

        auth_token = detail_datapacket.request.headers.get('auth-token')
        post_data = {
            'beginDate': begin_date,
            'endDate': end_date,
            **PostData.goods_ranking_export,
        }
        post_data_str, h_enc = generate__h_enc(
            auth_token, DataPacketUrls.export, post_data
        )
        headers = {
            **PUB_WEB_REQUEST_HEADERS,
            'H-Enc': h_enc,
            'Auth-Token': auth_token,
            'Content-Type': 'application/json',
        }

        page.change_mode('s', go=False)
        resp = page.post(
            DataPacketUrls.export,
            data=post_data_str,
            headers=headers,
            timeout=_timeout,
        )
        resp_data: dict = resp.json()
        if resp_data.get('code') != 20000:
            errmsg = resp_data.get('message')
            raise RuntimeError(f'表格下载出错: {errmsg}')

        download_url = resp_data.get('content')
        download_state, file_path = page.download(
            file_url=download_url,
            save_path=gettempdir(),
            file_exists='overwrite',
            timeout=_timeout,
            show_msg=False,
        )
        if download_state != 'success':
            raise RuntimeError('表格下载失败')

        page.change_mode('d', go=False)
        page.close()

        if raw is True:
            return file_path

        data_list = OsTools.xlsx_read(file_path=file_path)
        OsTools.file_remove(file_path)

        data_list = self.__format__data_list(data_list)
        return data_list

    def get__detail__from_favorite(
        self,
        date: str,
        raw=False,
        timeout: float = None,
    ):
        """
        获取来自收藏的商品排行表格数据

        Args:
            date: 指定日期
            raw: 为 True 则返回下载的文件路径, 否则返回表格数据
            timeout: 超时时间
        Returns:
            表格数据或表格文件的路径
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout
        page = self._browser.chromium.new_tab()

        def wait__detail_datapacket(callback: Callable, target_url: str = None):
            page.listen.start(
                targets=DataPacketUrls.detail if target_url is None else target_url,
                method='POST',
                res_type='XHR',
            )
            callback()
            datapacket = page.listen.wait(timeout=_timeout)
            return datapacket

        detail_datapacket = wait__detail_datapacket(lambda: page.get(Urls.ranking))
        if not detail_datapacket:
            raise TimeoutError('进入页面后获取表格详情数据包超时')

        def check__detail_total__by_datapacket():
            detail_datapacket__processor = DataPacketProcessor(detail_datapacket)
            detail_datapacket__data = detail_datapacket__processor.filter(
                '?content.total'
            )
            if not detail_datapacket__data.get('total'):
                raise ValueError('表格详情数据为空')

        check__detail_total__by_datapacket()

        # 切换到关注tab
        favorite_tab = page.ele('#rc-tabs-1-tab-2', timeout=3)
        if not favorite_tab:
            raise RuntimeError('未找到 [关注] 选项卡')

        detail_datapacket = wait__detail_datapacket(
            lambda: favorite_tab.click(by_js=True), DataPacketUrls.favorite_detail
        )
        if not detail_datapacket:
            raise TimeoutError('切换到 [关注] 选项卡后获取表格详情数据包超时')

        check__detail_total__by_datapacket()

        is_yeasterday = DateTimeTools.date_yesterday() == date
        if not is_yeasterday and not wait__detail_datapacket(
            lambda: pick__date(date, page)
        ):
            raise RuntimeError('修改日期后获取表格详情数据包超时')

        try:
            report_file_path = self._download_report(page, download_btn_index=1)
        except Exception as e:
            raise RuntimeError(f'下载报表失败: {e}') from e

        page.close()

        if raw is True:
            return report_file_path

        data_list = OsTools.xlsx_read(file_path=report_file_path)
        OsTools.file_remove(report_file_path)

        data_list = self.__format__data_list(data_list)
        return data_list

    def get__detail__from_favorite__by_api(
        self, begin_date: str, end_date: str, raw=False, timeout: float = None
    ):
        """
        通过 API 获取关注的商品排行表格数据

        Args:
            begin_date: 开始日期
            end_date: 结束日期
            raw: 为 True 则返回下载的文件路径, 否则返回表格数据
            timeout: 超时时间
        Returns:
            表格数据或表格文件的路径
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout

        page = self._browser.chromium.new_tab()
        page.listen.start(targets=DataPacketUrls.detail, method='POST', res_type='XHR')
        page.get(Urls.ranking)
        detail_datapacket = page.listen.wait(timeout=_timeout)
        if not detail_datapacket:
            raise TimeoutError('表格详情数据包获取超时')

        detail_datapacket__processor = DataPacketProcessor(detail_datapacket)
        detail_datapacket__data = detail_datapacket__processor.filter('?content.total')
        if not detail_datapacket__data.get('total'):
            raise ValueError('表格详情数据为空')

        auth_token = detail_datapacket.request.headers.get('auth-token')
        post_data = {
            'beginDate': begin_date,
            'endDate': end_date,
            **PostData.goods_ranking_export,
        }
        post_data_str, h_enc = generate__h_enc(
            auth_token, DataPacketUrls.favorite_export, post_data
        )
        headers = {
            **PUB_WEB_REQUEST_HEADERS,
            'H-Enc': h_enc,
            'Auth-Token': auth_token,
            'Content-Type': 'application/json',
        }

        page.change_mode('s', go=False)
        resp = page.post(
            DataPacketUrls.favorite_export,
            data=post_data_str,
            headers=headers,
            timeout=_timeout,
        )
        resp_data: dict = resp.json()
        if resp_data.get('code') != 20000:
            errmsg = resp_data.get('message')
            raise RuntimeError(f'表格下载出错: {errmsg}')

        download_url = resp_data.get('content')
        download_state, file_path = page.download(
            file_url=download_url,
            save_path=gettempdir(),
            file_exists='overwrite',
            timeout=_timeout,
            show_msg=False,
        )
        if download_state != 'success':
            raise RuntimeError('表格下载失败')

        page.change_mode('d', go=False)
        page.close()

        if raw is True:
            return file_path

        data_list = OsTools.xlsx_read(file_path=file_path)
        OsTools.file_remove(file_path)

        data_list = self.__format__data_list(data_list)
        return data_list



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-05-19
Author: Martian Bugs
Description: 数据采集器
"""

from BrowserAutomationLauncher import BrowserInitOptions, Launcher

from ._login import Login
from .goods_analysis.goods_analysis import GoodsAnalysis


class Collector:
    """采集器. 使用之前请先调用 `connect_browser` 方法连接浏览器."""

    def __init__(self):
        self._browser_launcher = Launcher()
        self._goods_analysis = None

    def connect_browser(self, port: int):
        """
        连接浏览器

        Args:
            port: 浏览器调试端口号
        """

        browser_init_options = BrowserInitOptions()
        browser_init_options.set_basic_options(port=port)
        self._browser = self._browser_launcher.init_browser(browser_init_options)

        self._login_utils = Login(self._browser)

    def login_by_password(self, username: str, password: str):
        """
        通过 账号密码 登录账号

        Args:
            username: 账号
            password: 密码
        """

        return self._login_utils.by_password(username, password)

    @property
    def goods_analysis(self):
        """商品分析模块"""

        if not self._goods_analysis:
            self._goods_analysis = GoodsAnalysis(self._browser)
        return self._goods_analysis
